% This matlab program uses the modified EM of 'A Modified EM Algorithum
% for estimation in Generalized Mixed Models' Steel(1996) Biometric, 52,
% pp1295_1310. It finds estimates of the parameters alpha, beta, gamma 
% and the variance component sigmasq for the binomial GLMM.
% The binomial GLMM is defined as pi=1/(1+exp(-eta)) where
% eta = alpha + beta*m + gamma*r + b
% m is the number of strawberry flowers at the previous rank
% r is the rank of the straweberry flower
% b is the random inflorescence effect b ~ N(0,sigmasq)

% The data is stored in the matrix data
% The first coloumn is Y the number of strawberry flowers
% The second coulmn is m
% The thrid column is r
% The rest of the coulmns give U, the design matrix for the random effects

sizeData=size(Data);
n=sizeData(1);
Y=Data(:,1);             % Y is the vector of responses. Ie the number of
                         % strawberries at that level
X1=ones(sizeData(1),1);
X=[X1 Data(:,2:3)];           % X is the matrix of covariates.
sizeX=size(X);
M=2*X(:,2);              % Y(i)~Bin(M(i),pi(i))
U=Data(:,4:sizeData(2)); % U is the matrix of coefficents of random
                         % effects
sizeU=size(U);
beta0 =input('Input starting values [alpha beta gamma]');
sigmasq=input('Input starting value of sigmasq');
b0=randn(sizeU(2),1)*sqrt(sigmasq);   % This is the starting value for b,
                                      % the vector of random effects.
beta0=beta0';

alpha=beta0(1);
beta=beta0(2);
gamma=beta0(3);
sigma2=sigmasq;

alphapr=100;
betapr=100;
gammapr=100;
sigmapr=100;

II=diag(ones(sizeU(2),1));
I2=diag(ones(sizeX(1),1));
grad=10000;

while (alphapr~=alpha) | (betapr~=beta) | (gammapr~=gamma) | (sigmapr~=sigma2)
   alphapr=alpha;
   betapr=beta;
   gammapr=gamma;
   sigmapr=sigma2;

%  E1 step
   for t=1:5
      neta0=X*beta0+U*b0;
      pi0=1./(1+exp(-neta0));
      muhat=M.*pi0;
      W=diag(M.*pi0.*(1-pi0));
      V=U'*W*U+II*sigmasq^(-1);
      b0=b0+inv(V)*(U'*(Y-muhat)-II*sigmasq^(-1)*b0);
    end

%  E2 step
   UU=diag(M.*(1-2*pi0).*pi0.*(1-pi0));
   invV=inv(V);
   Eb0Tb0=b0'*b0+trace(invV*(II-U'*diag(UU*U*invV*b0)*U));

%  M step
   sigmasq=Eb0Tb0/sizeU(2);
   for i=1:5
      neta0=X*beta0+U*b0;
      pi0=1./(1+exp(-neta0));
      muhat=M.*pi0;
      W=diag(M.*pi0.*(1-pi0));
      V=U'*W*U+II*sigmasq^(-1);
      invV=inv(V);
      EXTmu1=X(:,1)'*muhat+trace(invV*U'*diag(UU*(I2-U*invV*U'*W)*X(:,1))*U)/2;
      EXTmu2=X(:,2)'*muhat+trace(invV*U'*diag(UU*(I2-U*invV*U'*W)*X(:,2))*U)/2;
      EXTmu3=X(:,3)'*muhat+trace(invV*U'*diag(UU*(I2-U*invV*U'*W)*X(:,3))*U)/2;
      EXT=[EXTmu1 EXTmu2 EXTmu3]';
      neta0=X*beta0+U*b0;
      pi0=1./(1+exp(-neta0));
      W=diag(M.*pi0.*(1-pi0));
      beta0=beta0+inv(X'*W*X)*(X'*Y-EXT);
   end
   alpha=round(beta0(1)*100)/100;
   beta=round(beta0(2)*100)/100;
   gamma=round(beta0(3)*100)/100;
   sigma2=round(sigmasq*100)/100;
end
[alpha beta gamma sigma]
